from genomfart.parsers.AGPmap import AGPMap
from genomfart.utils.bigDataFrame import BigDataFrame
from numba import jit
import numba as nb
import numpy as np
import re

@jit(argtypes=(
    nb.int64[:], nb.int64[:], nb.double[:], nb.double[:,:], nb.int64,
    nb.int64, nb.int64, nb.double), restype=nb.void)
def _projectSnpBoolean(parents, popIndex, snpvalues, genotypes, nSamples,
                       leftmarker, rightmarker, pd):
    """ Projects SNPs if parents have boolean genotypes
    """
    for i in xrange(nSamples):
        if parents[popIndex[i]] == 0:
            snpvalues[i] = 0
        else:
            leftval = genotypes[i,leftmarker]
            rightval = genotypes[i,rightmarker]
            if (leftval == rightval):
                snpvalues[i] = leftval
            else:
                snpvalues[i] = leftval*(1-pd) + rightval*pd

class snp_projector:
    """ Class used to project SNPs from a set of founders onto
    descendants
    """
    def __init__(self, chromosome, chrom_length, mapFile, rilFile, useAgpV2=False):
        """ Instantiates the projector

        Parameters
        ----------
        chromosome : int
            Chromosome to project onto
        chrom_length : int
            Length of the chromosome
        mapFile : str
            The filename for the map
        rilFile : str
            The filename for the RIL file
        useAgpV2 : boolean
            Whether this is version 2 of the AGPmap format        
        """
        self.chromosome = chromosome
        self.chrom_length = chrom_length
        self.theAGPMap = AGPMap(mapFile, useAgpV2)
        self.firstMarker = None
        self.sampleNameMap = {}
        self.genotypes = []
        self.samp_names = []        
        self.importMarkersForMap(rilFile)
        self.maxMarker = self.genotypes.shape[1]-1
    def importMarkersForMap(self, rilFile):
        """ Reads the data file for a chromosome with the sample allele states

        Parameters
        ----------
        rilFile : str
            The filename for the RIL file
        """
        rilFile = BigDataFrame(rilFile, header=True, assume_uniform_types=False)
        nGenos = len(rilFile.colLabels)-5
        self.genotypes = [[] for x in xrange(nGenos)]
        self.sampleNameMap = dict((k,v-5) for k,v in rilFile.colLabels.items() if v >= 5)
        self.samp_names = sorted(self.sampleNameMap.keys(),
                                 key=lambda x: self.sampleNameMap[x])
        count = 0
        nMarkers = 0
        self.firstMarker = self.theAGPMap.getMarkerNumber(self.theAGPMap.getFirstMarkerName(self.chromosome))
        for row in rilFile.iterrows(cache=False):
            for samp in self.samp_names:
                self.genotypes[self.sampleNameMap[samp]].append(row[samp])
        self.genotypes = np.array(self.genotypes)
        # header = rilFile.readline().strip().split('\t')
        # self.firstMarker = int(re.search('\d+',header[2]).group())
        # nMarkers = len(header)-1
        # count = 0
        # while 1:
        #     data = rilFile.readline().strip()
        #     if data == '': break
        #     data = data.split('\t')
        #     self.sampleNameMap[data[0]] = count
        #     self.genotypes.append([])
        #     self.samp_names.append(data[0])
        #     for j in xrange(nMarkers):
        #         self.genotypes[-1].append(float(data[j+1]))
        # self.genotypes = np.array(self.genotypes)
        # rilFile.close()
    def projectSnpBoolean(self, parents, pos, chrom_length, popIndex):
        """ Projects a SNP onto descendants if parent values are boolean

        Parameters
        ----------
        parents : np.ndarray, boolean
            Array of parent genotypes
        pos : int
            Position of the SNP
        popIndex : np.ndarray, int
            Indices of the population for each sample
        """
        left_mark, left_mark_pos, right_mark, right_mark_pos = self.theAGPMap.getInterval(self.chromosome,
                                                                                          pos)
        left = left_mark_pos
        if left is None:
            left = 0
        right = right_mark_pos
        if right is None:
            right = self.chrom_length
        # Proportion of distance of SNP between left and right markers
        pd = 0.
        if right != left:
            pd = (float(pos-left))/(float(right-left))
        leftmarker = 0
        if left_mark:
            leftmarker = self.theAGPMap.getMarkerNumber(left_mark)
            leftmarker = leftmarker-self.firstMarker+1
        rightmarker = self.maxMarker
        if right_mark:
            rightmarker = self.theAGPMap.getMarkerNumber(right_mark)
            rightmarker = rightmarker - self.firstMarker + 1
        nSamples = len(popIndex)
        snpvalues = np.zeros(nSamples)
        _projectSnpBoolean(parents, popIndex, snpvalues, self.genotypes,
                           nSamples, leftmarker, rightmarker, pd)
        return snpvalues
    def projectAllSnps(self, chrom_length, popIndex, founder_data, boolean = True,
                       positions = None):
        """ Projects all SNPs on a chromsome onto descendants

        Parameters
        ----------
        chrom_length : int
            The length of the chromosome
        popIndex : np.ndarray, int
            Indices of the population for each sample
        founder_data : genomfart.parsers.SNPdata
            Object containing the founder SNP data
        boolean : boolean
            Whether the parents returned have boolean values for SNPs
        positions : set of ints
            A set of specific positions to project. If None, all SNPs will
            be projected

        Returns
        -------
        Generator non-reference allele count for each RIL, as an array of
        doubles

        Examples
        --------
        >>> from genomfart.utils.snp_projector import snp_projector
        >>> from genomfart.parsers.SNPdata import SNPdata
        >>> import numpy as np
        >>> import re
        >>> import os
        >>> chrom = 10
        >>> chrom_length = 149686046
        >>> home_dir = os.path.expanduser('~')
        >>> map_dir = os.path.join(home_dir,'Documents','Zea','Data','Sequence',
        ... 'NAM_imputed_maps','imputedMarkers.0.1cm')
        >>> founder_dir = os.path.join(home_dir,'Documents','Zea','Data','Sequence',
        ... 'NAM_founder_genos')
        >>> founder_file = os.path.join(founder_dir,
        ... 'gwas_snps_union_chr%d.h1.h2_minors_indels.cnv_genic_2kwin_500bpbin.20130605.txt' % chrom)
        >>> map_file = os.path.join(map_dir,'Imputed_0.1cm_master.txt')
        >>> ril_file = os.path.join(map_dir,'Imputed_0.1cm_chr%d_clean.txt' % chrom)
        >>> projector = snp_projector(chrom,chrom_length,map_file,ril_file)
        >>> popIndex = np.array(map(lambda x: (17 if x[0] == 'M' else int(re.search('(?<=Z)\d+(?=E)',x).group()))-1,)
        ... projector.samp_names), dtype=np.int64)
        >>> founder_data = SNPdata(chrom, founder_file, 'B73')
        >>> projection = projector.projectAllSnps(chrom_length, popIndex, founder_data)
        """
        if not boolean:
            raise NotImplementedError("Non-boolean projection not yet implemented")
        founder_data.reset()
        while (founder_data.next()):
            if positions is not None:
                if founder_data.getPosition() not in positions: continue
            if boolean:
                parents = np.array(founder_data.getGenotype(), dtype=np.int64)
                yield self.projectSnpBoolean(parents, founder_data.getPosition(),
                        chrom_length, popIndex)
            else:
                pass

        



import unittest
from genomfart.parsers.vcf import VCF_parser
from genomfart.data.data_constants import VCF_TEST_FILE

debug = False

class vcfTest(unittest.TestCase):
    """ Unit tests for vcf.py """
    @classmethod
    def setUpClass(cls):
        cls.parser = VCF_parser(VCF_TEST_FILE)
    def test_parse_geno_depths(self):
        if debug: print("Testing parse_geno_depths")
        depth_parser = self.parser.parse_geno_depths()
        chrom,pos,allels,depths = next(depth_parser)
        self.assertEqual(sum(depths['COSTICH_2014']),1)
        self.assertEquals(sum(depths['FL_05_15_1']),0)
        chrom,pos,allels,depths = next(depth_parser)
        chrom,pos,allels,depths = next(depth_parser)
        self.assertEqual(depths['COSTICH_2014'],(2,1))
        self.assertEquals(sum(depths['FL_05_15_1']),0)
    def test_parse_select_geno_depths(self):
        if debug: print("Testing parse_select_geno_depths")
        depth_parser = self.parser.parse_select_geno_depths(['COSTICH_2014','FL_34798'],
                                                            info_dict=True, start=100,
                                                            end=130)
        chrom,pos,alleles,depths,info = next(depth_parser)
        self.assertEqual(len(depths), 2)
        self.assertEqual(pos, 105)
        self.assertEqual(depths['COSTICH_2014'],(9,2))
        self.assertEqual(depths['FL_34798'],(1,0))
        chrom,pos,alleles,depths,info = next(depth_parser)
        self.assertEqual(pos, 124)
        chrom,pos,alleles,depths,info = next(depth_parser)
        with self.assertRaises(StopIteration):
            next(depth_parser)
    def test_parse_select_geno_generic(self):
        if debug: print("Testing parse_select_geno_generic")
        geno_parser = self.parser.parse_select_geno_generic(['COSTICH_2014','FL_05_15_1','FL_34798'],
                                                            info_dict=True, start=100,
                                                            end=130)
        chrom,pos,alleles,genos,info = next(geno_parser)
        self.assertEqual(genos['COSTICH_2014']['GT'], '0/1')
        self.assertEqual(genos['COSTICH_2014']['AD'], '9,2')
        self.assertEqual(genos['FL_05_15_1']['GT'], './.')
        with self.assertRaises(KeyError):
            genos['FL_05_15_1']['AD']
        chrom,pos,alleles,gemps,info = next(geno_parser)
        self.assertEqual(len(genos), 3)
        self.assertEqual(pos, 124)
        chrom,pos,alleles,genos,info = next(geno_parser)
        with self.assertRaises(StopIteration):
            next(geno_parser)
    def test_parse_site_infos(self):
        if debug: print("Testing parse_site_infos")
        info_parser = self.parser.parse_site_infos()
        chrom,pos,alleles,info = next(info_parser)
        self.assertEqual(pos, 13)
        self.assertEqual(info['GN'],(2,1,0))
        self.assertEqual(info['AQ'],(36,33))
    def test_get_affected_ref_bases(self):
        if debug: print("Testing get_affected_ref_bases")
        self.assertEqual(VCF_parser.get_affected_ref_bases(20,'C','T'),set([20]))
        self.assertEqual(VCF_parser.get_affected_ref_bases(20,'C','CTAG'), set([]))
        self.assertEqual(VCF_parser.get_affected_ref_bases(20,'TCG','T'), set([21,22]))
        self.assertEqual(VCF_parser.get_affected_ref_bases(20,'TCGCG','TCG'),set([24,23]))
        self.assertEqual(VCF_parser.get_affected_ref_bases(20,'TCGCG','TCGCGCG'),set([]))
    def test_get_substituted_ref_bases(self):
        if debug: print("Testing get_substituted_ref_bases")
        self.assertEqual(VCF_parser.get_substituted_ref_bases(20,'C','T'),set([20]))
        self.assertEqual(VCF_parser.get_substituted_ref_bases(20,'C','CTAG'), set([]))
        self.assertEqual(VCF_parser.get_substituted_ref_bases(20,'TCG','T'), set([]))
        self.assertEqual(VCF_parser.get_substituted_ref_bases(20,'TCGCG','TCG'),set([]))
        self.assertEqual(VCF_parser.get_substituted_ref_bases(20,'TCGCG','TCGGCGCG'),set([24,23]))        
        self.assertEqual(VCF_parser.get_substituted_ref_bases(20,'TCGCG','TCGCGCG'),set([]))
    def test_get_nw_aligned_alleles(self):
        if debug: print("Testing get_nw_aligned_alleles")
        ref = 'GAAGGTTGCTTCTCTAGAAGGCAATTTGGACTCAACAACCCCTTCAATCAAGCCAATGTATGGTGCCCAATCG'
        alt = 'GCAAGGTTGCTTCTCTCGAAGGCAATTTGGGCTTAACAACCCCCTTGAATCAAGCCTGTGCATGATGTCCAAGT'
        ref_aln,alt_aln,score = VCF_parser.get_nw_aligned_alleles(ref, alt)
        self.assertEqual(ref_aln, 'G-AAGGTTGCTTCTCTAGAAGGCAATTTGGACTCAACAACCCC-TTCAATCAAGCCAATGTATGGTGCCCAATCG-')
        self.assertEqual(alt_aln, 'GCAAGGTTGCTTCTCTCGAAGGCAATTTGGGCTTAACAACCCCCTTGAATCAAGCCTGTGCATGATGTCCAA--GT')
        self.assertEqual(score, 27.)
        ref_aln,alt_aln,score = VCF_parser.get_nw_aligned_alleles(ref, 'G')
        self.assertEqual(ref_aln, 'GAAGGTTGCTTCTCTAGAAGGCAATTTGGACTCAACAACCCCTTCAATCAAGCCAATGTATGGTGCCCAATCG')
        self.assertEqual(alt_aln, 'G------------------------------------------------------------------------')
        self.assertEqual(score, -74.)
        ref_aln,alt_aln,score = VCF_parser.get_nw_aligned_alleles('G', alt)
        self.assertEqual(ref_aln, 'G-------------------------------------------------------------------------')
        self.assertEqual(alt_aln, 'GCAAGGTTGCTTCTCTCGAAGGCAATTTGGGCTTAACAACCCCCTTGAATCAAGCCTGTGCATGATGTCCAAGT')
        self.assertEqual(score, -75.)
        ref_aln,alt_aln,score = VCF_parser.get_nw_aligned_alleles(ref, 'A')
        self.assertEqual(ref_aln, 'GAAGGTTGCTTCTCTAGAAGGCAATTTGGACTCAACAACCCCTTCAATCAAGCCAATGTATGGTGCCCAATCG')
        self.assertEqual(alt_aln, 'A------------------------------------------------------------------------')
        self.assertEqual(score, -77.)
        ref_aln,alt_aln,score = VCF_parser.get_nw_aligned_alleles('A', alt)
        self.assertEqual(ref_aln, 'A-------------------------------------------------------------------------')
        self.assertEqual(alt_aln, 'GCAAGGTTGCTTCTCTCGAAGGCAATTTGGGCTTAACAACCCCCTTGAATCAAGCCTGTGCATGATGTCCAAGT')
        self.assertEqual(score, -78.)
    def test_get_substituted_ref_bases_nw(self):
        if debug: print("Testing get_substituted_ref_bases_nw")
        self.assertEqual(VCF_parser.get_substituted_ref_bases_nw(20,'C','T'),set([20]))
        self.assertEqual(VCF_parser.get_substituted_ref_bases_nw(20,'C','CTAG'), set([]))
        self.assertEqual(VCF_parser.get_substituted_ref_bases_nw(20,'TCG','T'), set([]))
        self.assertEqual(VCF_parser.get_substituted_ref_bases_nw(20,'TCGCG','TCG'),set([]))
        self.assertEqual(VCF_parser.get_substituted_ref_bases_nw(20,'TCGCG','TCGGCGCG'),set([]))        
        self.assertEqual(VCF_parser.get_substituted_ref_bases_nw(20,'TCGCG','TCGCGCG'),set([]))        
if __name__ == "__main__":
    debug = True
    unittest.main(exit = False)



import unittest
from genomfart.utils.version_mapper import version_mapper
from genomfart.data.data_constants import VERSION_TEST_FILE

debug = False

class version_mapper_test(unittest.TestCase):
    """ Tests for version_mapper.py """
    @classmethod
    def setUpClass(cls):
        cls.mapper = version_mapper(VERSION_TEST_FILE)
    def test_v1_to_v2_map(self):
        if debug: print("Testing v1_to_v2_map")
        # Cis direction
        chrom,pos,orient = self.mapper.v1_to_v2_map(1, 13857655)
        self.assertEqual((chrom,pos,orient),(1,13856155,1))
        # Trans direction
        chrom,pos,orient = self.mapper.v1_to_v2_map(10,139836890)
        self.assertEquals((chrom,pos,orient),(2,16760272,'-'))
    def test_v2_to_v1_map(self):
        if debug: print("Testing v2_to_v1_map")
        # Cis direction
        chrom,pos,orient = self.mapper.v2_to_v1_map(1,13856155)
        self.assertEqual((chrom,pos,orient),(1, 13857655,1))
        # Trans direction
        chrom,pos,orient = self.mapper.v2_to_v1_map(2,16760272)
        self.assertEquals((chrom,pos,orient),(10,139836890,'-'))
    def test_v1_to_v2_seg_map(self):
        if debug: print("Testing v1_to_v2_seg_map")
        # Cis direction    
        seg_map = self.mapper.v1_to_v2_seg_map(1, 13857655, 13857659)
        self.assertEqual(list(seg_map.values())[0],
            (1, 13856155, 13856159, 1))
        # Trans direction
        seg_map = self.mapper.v1_to_v2_seg_map(10,139836890,139836895)
        self.assertEqual(list(seg_map.values())[0],
            (2,16760272,16760267,'-')
        )
    def test_v2_to_v1_seg_map(self):
        if debug: print("Testing v2_to_v1_seg_map")
        # Cis direction    
        # seg_map = self.mapper.v1_to_v2_seg_map(1, 13857655, 13857659)
        # self.assertEqual(list(seg_map.values())[0],
        #     (1, 13856155, 13856159, 1))
        seg_map = self.mapper.v2_to_v1_seg_map(1, 13856155, 13856159)
        self.assertEqual(list(seg_map.values())[0],
                         (1, 13857655, 13857659, 1))
        # Trans direction
        # seg_map = self.mapper.v1_to_v2_seg_map(10,139836890,139836895)
        # self.assertEqual(list(seg_map.values())[0],
        #     (2,16760272,16760267,'-')
        # )
        seg_map = self.mapper.v2_to_v1_seg_map(2,16760267,16760272)
        self.assertEqual(list(seg_map.values())[0],
                         (10,139836895,139836890,'-'))
if __name__ == '__main__':
    debug = True
    unittest.main(exit = False)



#!/usr/bin/env python
from genomfart.utils.bigDataFrame import BigDataFrame
import numpy as np
import argparse
import os

def shuffle_pheno_within_family(in_file, out_file):
    """
    Shuffle a phenotype within each family, and write to file

    Parameters
    ----------
    in_file : str
        Name of the PLINK-formatted phenotype file
    out_file : str
        Name to give the output file
    """
    # Open up file to shuffle
    frame = BigDataFrame(in_file, header=False, assume_uniform_types=False)
    ## Create dictionary of family -> values
    pheno_vals = {}
    for row in frame:
        if row[0] not in pheno_vals:
            pheno_vals[row[0]] = []
        pheno_vals[row[0]].append(row[2])
    ## Make permuted file
    write_file = open(out_file,'w')
    count = 0
    for fam, vals in pheno_vals.iteritems():
        np.random.shuffle(vals)
        for val in vals:
            write_line = [str(fam),str(count),"%0.8g" % val if type(val) != str else val]
            write_file.write('\t'.join(write_line)+'\n')
            count += 1
    write_file.close()

def main():
    parser = argparse.ArgumentParser(description="Shuffle a phenotype within family in a PLINK-phenotype file")
    parser.add_argument('in_file',metavar='in_file',nargs='?',help='Input file (PLINK format)')
    parser.add_argument('out_file',metavar='out_file',nargs='?',help='Output file (PLINK format)')
    args = parser.parse_args()
    shuffle_pheno_within_family(args.in_file,args.out_file)

if __name__ == "__main__":
    main()



from genomfart.utils.genomeAnnotationGraph import genomeAnnotationGraph
import gzip

class gff_parser(genomeAnnotationGraph):
    """ Class used to parse and analyze GFF (version 3) files.
    The class represents any hierarchical structure as a directed graph
    and puts the individual pieces into a RangeBucketMap

    All coordinates are 1-based

    Examples
    --------

    >>> from genomfart.parsers.gff import gff_parser
    >>> from genomfart.data.data_constants import GFF_TEST_FILE
    >>> parser = gff_parser(GFF_TEST_FILE)
    >>> parser.get_overlapping_element_ids('Pt',100,4000)
    set(['three_prime_UTR:Pt_3363_4490:-', 'exon:Pt_1674_3308:-',
    'CDS:GRMZM5G836994_P01', 'exon:Pt_3363_4708:-',
    'transcript:GRMZM5G811749_T01', 'transcript:GRMZM5G836994_T01',
    'repeat_region:Pt_3550_3560:?', 'repeat_region:Pt_3683_3696:?',
    'gene:GRMZM5G811749', 'gene:GRMZM5G836994', 'repeat_region:Pt_320_1262:+',
    'repeat_region:Pt_3764_3775:?'])
    >>> gene_info = parser.get_element_info('gene:GRMZM5G811749')
    >>> gene_info
    {'Ranges': [[3363 , 5604]], 'attributes': [{'logic_name': 'genebuilder',
    'external_name': 'RPS16',
    'description': '30S ribosomal protein S16%2C chloroplastic  [Source:UniProtKB/Swiss-Prot%3BAcc:P27723]', 'ID': 'gene:GRMZM5G811749',
    'biotype': 'protein_coding'}], 'seqid': 'Pt', 'type': 'gene', 'strand': '-'}
    >>> [x for x in parser.get_element_ids_of_type('Pt','gene',start=100,end=4000)]
    ['gene:GRMZM5G836994', 'gene:GRMZM5G811749']
    """
    def __init__(self, gff_file, exclude_types = None):
        """ Instantiates the gff file

        Parameters
        ----------
        gff_file : str
            The filename of a gff file
        exclude_types : set
            The names of types (e.g. 'repeat') that you don't want to store

        Raises
        ------
        IOError
            If the file isn't correctly formatted
        """
        super(gff_parser, self).__init__()
        if exclude_types is None: exclude_types = set()
        if gff_file.endswith('.gz'):
            gff_handle = gzip.open(gff_file)
        else:
            gff_handle = open(gff_file)
        ## Parse the file
        with gff_handle:
            for line in gff_handle:
                if line.startswith('#'): continue
                elif len(line) < 2: continue
                line = line.strip().split('\t')
                if len(line) != 9:
                    raise IOError("Line(s) do not conform to gff v. 3 format")
                if line[2] in exclude_types: continue
                # Make a new RangeBucketMap for the seqid if necessary
                seqid = line[0]
                # Parse the attributes into a dictionary of key->val
                attr_dict = dict((k,v) for k,v in map(lambda x: x.split('='),
                                                      line[8].split(';')))
                # Check if the element has an ID. If not, give it one
                if 'ID' in attr_dict:
                    element_id = attr_dict['ID']
                else:
                    element_id = '%s:%s_%s_%s:%s' % (line[2],line[0],line[3],line[4],
                                                     line[6])
                parents = set()
                if 'Parent' in attr_dict:
                    parents = parents.union(attr_dict['Parent'].split(','))
                self.add_annotation(element_id, line[0], int(line[3]), int(line[4]),
                                    line[2], strand=line[6], parents=parents,
                                    **attr_dict)




from genomfart.utils.snp_projector import snp_projector
from genomfart.parsers.SNPdata import SNPdata
import numpy as np
import numba as nb
from numba import jit

@jit(argtypes=(
    nb.double[:], nb.double[:,:], nb.double, nb.int64
), restype=nb.boolean)
def _append_genetic_relationship(genotypes, relationship_mat, min_MAF, taxa_count):
    """ Calculates portion of genetic relationship accounted for by
    a single locus. Adds onto current matrix of genetic relationships

    Parameters
    ----------
    genotypes : np.ndarray, float
        Genotypes for the samples (between 0 and 2)
    relationship_mat : np.ndarray, float, square
        Matrix of current genetic relationships
    min_MAF : float
        Minimum minor allele frequence allowed for a marker
    taxa_count : int
        Number of taxa in matrix

    Returns
    -------
    Whether SNP was included
    """
    # Calculate p for this locus
    p = 0.
    for i in xrange(taxa_count):
        p += genotypes[i]
    p /= (2*taxa_count)
    if (p < min_MAF or (1.-p) < min_MAF):
        return False
    E_geno = 2*p
    V_geno = 2*p*(1-p)
    # Get genetic relationship values
    for j in xrange(taxa_count):
        for k in xrange(j+1):
            relationship_mat[j,k] += ((genotypes[j]-E_geno)*(genotypes[k]-E_geno))/V_geno
    return True

class genetic_relationship:
    """ Class used to calculate pairwise genetic relationships between taxa
    """
    @staticmethod
    def get_genetic_relationships(snp_generator, min_MAF=0.025, verbosity = None):
        """ Gets the genetic relationship matrix, as defined in the GCTA paper
        (e.g. eq. 3 from http://www.ncbi.nlm.nih.gov/pmc/articles/PMC3014363/)

        Parameters
        ----------
        snp_generator : generator
            Generator of numpy arrays of double, giving the genotype of each
            sample as a number between 0 and 2.
        min_MAF : float
            Minimum minor allele frequency for a marker
        verbosity : int, optional
            If not None, how often to print to screen how many snps have
            been processed
        Returns
        -------
        (Numpy square array, in which the lower diagonal contains the
        genetic relationship values. Order of rows and columns is the same
        as in the generator), number of markers used
        """
        relationship_array = None
        taxa_count = 0
        marker_count = 0
        iter_count = 0
        while 1:
            try:
                genotypes = next(snp_generator)
                if relationship_array is None:
                    taxa_count = len(genotypes)
                    relationship_array = np.zeros((taxa_count, taxa_count))
                included = _append_genetic_relationship(genotypes, relationship_array,
                            min_MAF, taxa_count)
                if included:
                    marker_count += 1
                iter_count += 1
                if verbosity:
                    if iter_count % verbosity == 0:
                        print "%d markers processed..." % iter_count
                        print (np.max(relationship_array), np.min(relationship_array),
                               np.any(np.isnan(relationship_array)))
            except StopIteration:
                break
            except ValueError:
                continue
        return (1./marker_count)*relationship_array, marker_count
        
